%% Clean up
clear all

%% Set up variables and data
test_function = @myDetectionFunction;

load('Snapshot2.mat')

%% Run the actual test

% Obtained by inspection:
% For our tests we're more interested in the min and max rather than min
% and width.
bounds = [299 215 347 296
          252 126 295 221];

bbox_task = test_function(I,D);

bbox_task(:,3:4) = bbox_task(:,3:4) + bbox_task(:,1:2);

%% Because we can't guarantee the order of the bounding boxes we'll align them
N = size(bounds,1);
C = size(bbox_task, 1);
num_cans = N;

bounds_use = bounds;
bbox_use = bbox_task;

extra_cans = 0;

if (C < N)
    % The function didn't pick up all the possible cans
    bbox_use = [bbox_task; inf(N-C,4)];
    C=N;
elseif (C > N)
    bounds_use = [bounds; inf(C-N,4)];
    extra_cans = C-N;
    N=C;
end

ind = 1:N;
perm = perms(ind);
P = size(perm,1);
% Each row is the error of different bounding boxes
% Each column is the error for each permutation:
err = zeros(N,P);

% The threshold defines how much error we will allow in each border before
% dropping the mark. The amount that you are outside this threshold is used
% to determine the mark.
threshold = 5;
%%
for i=1:P
    % Reorder the boxes
    bounds_order = bounds_use(perm(i,:),:);
    % Get the errors of each border
    border_error = abs(double(bounds_order) - double(bbox_use));
    % Threshold these errors
    border_error(border_error <= threshold) = 0;
    % Select the ones to reduce
    select = border_error > threshold;
    border_error(select) = border_error(select) - threshold;
    
    % Square and sum the border errors
    err(:,i) = sum(border_error.^2, 2);
end


%% Sum the error at each permuationd_sum = distance_sqr;
err_filt = err;
err_filt(err_filt==inf) = 0;
err_sum = sum(err_filt);

% Locate the smallest error
[err_min err_ind] = min(err_sum);

% Your mark for the test will be split into as many parts as there are
% cans. Say there are 5 cans in the scene, then each can is worth 20% of
% that test. Then the error of each coke can affects each part of the mark.
errors = err(:,err_ind);

%% Now let's actually figure out the mark
% This value is then scaled so that you get half-marks for being 10 pixels
% out per boarder.

half_error = (10^2)*4;
marks = 1.0 - 0.5*(errors/half_error);
% Zero is the lowest mark
marks(marks<0) = 0;

test_mark = mean(marks)


