%% Clean up
clear all

%% Set up variables and data
test_function = @myLocationFunction;

load('Basic_2.mat')
d = double(d);

imshow(rgb)
figure
imshow(d/max(d(:)))

%% Run the actual test

% Obtained by inspection:
% This is the location of the top of the can

coords = test_function(rgb,d);

%% Check how close...
N = size(positions,1);
C = size(coords, 1);

num_cans = N;

pos_use = positions;
coords_use = coords;

extra_cans = 0;

if (C < N)
    % The function didn't pick up all the possible cans
    coords_use = [coords; inf(N-C,3)];
    mask(C+1:end) = false;
    C=N;
elseif (C > N)
    pos_use = [positions; inf(C-N,3)];
    mask(N+1:end) = false;
    extra_cans = C-N;
    N=C;
end

ind = 1:N;
perm = perms(ind);
P = size(perm,1);
% Each row is the error of different bounding boxes
% Each column is the error for each permutation:
distance_sqr = zeros(N,P);
rot_err_sqr = zeros(N,P);

% The threshold defines how much error we will allow before dropping the
% mark. The amount that you are outside this threshold is used to determine
% the mark.
threshold_distance = 30; %mm

for i=1:P
    % Reorder the actual positions
    pos = pos_use(perm(i,:),:);
    % Get the DISTANCES for each point
    dp = sum((pos(:,1:3) - coords_use(:,1:3)).^2, 2);
    dp = sqrt(dp);
    
    % We know that dp will be positive
    dp(dp < threshold_distance) = 0;
    % Cut down the ones that are above the threshold
    select = dp >= threshold_distance;
    dp(select) = dp(select) - threshold_distance;
    
    distance_sqr(:,i) = dp.^2;
    
    % This basically allows the 180 deg errors in rotation. This is to say,
    % if the can rotation is 10 deg, this will also allow -170, 190, 370,
    % etc...
    dr = mod(pos(:,4) - coords_use(:,4), 180);
    dr(dr>90) = (dr(dr>90) - 180);
    
    rot_err_sqr(:,i) = dr.^2;
end

%% Sum the error at each permutation
d_sum = distance_sqr;
d_sum(d_sum==inf) = 0;
err_sum = sum(d_sum);
% Locate the permutation with the lowest error
[err_min err_ind] = min(err_sum);

% Your mark for the test will be split into parts. The number of parts is:
% max(number of cans that you detects, number of cans in the scene)
%
% Say there are 5 cans in the scene, then each can is worth 20% of
% that test. Then the error of each coke can affects each part of the mark.
%
% Also NOTE! That if you detected 10 cans in that same scene, each can that
% is "correct" will now only be worth 10%
distance_errors = distance_sqr(:,err_ind);
rotation_errors = rot_err_sqr(:,err_ind);

%% Now let's actually figure out the mark
% This value is then scaled so that you get half-marks for being just over 
% a can's width away from the result.

half_distance_error = 70^2;
dist_marks = 1.0 - 0.5*(distance_errors/half_distance_error);
% Zero is the lowest mark
dist_marks(dist_marks<0) = 0;

half_rot_error = 10^2;
rot_marks = 1.0 - 0.5*(rotation_errors/half_rot_error);
rot_marks(rot_marks<0) = 0;

% Your distance result is 75% of the marks
test_mark = mean(0.75*dist_marks + 0.25*rot_marks)
